<?php

namespace ShirtigoCockpitPrintfulfillment\Components;

use Doctrine\Common\Util\Debug;
use Doctrine\ORM\Mapping\Entity;
use League\Flysystem\FileNotFoundException;
use Shopware\Components\HttpClient\RequestException;
use Shopware\Components\Logger;
use Shopware\Models\Log\Log;
use Shopware\Models\Order\Order;
use Shopware\Models\User\User;

class FulfillmentCreatorComponent
{

    private $user;

    private $reference;

    const referenceFile='/.reference';

    public function __construct(User $user) {

        $this->user=$user;

        if (!file_exists(__DIR__.self::referenceFile)) {
            throw new FileNotFoundException('The reference to Shirtigo Cockpit was not found. Please fill out the config-form in the plugin and press the save-button.');
        }

        $this->reference=file_get_contents(__DIR__.self::referenceFile);
    }


	/**
	 * Update the order-status in Shirtigo Cockpit.
	 *
	 * @param array $data
	 *
	 * @return bool
	 */
    public function createShirtigoFulfillment(array $data) {

        // We better transmit the whole order-data to Cockpit here to get more independent
        // of (possible) future changes in Shopware.

        $url=Config::cockpitURL."/api/webhooks/$this->reference/fulfillments/create";

        try {

	        $this->createAuthorizedPostRequest( $url, $data );

        } catch (RequestException $exception) {

        	return false;
        }

        return true;

    }

    private function createAuthorizedPostRequest( $url, array $data) {

        $ch=curl_init($url);

        $jsonRequestData=json_encode($data);

        $checksum=md5($this->user->getApiKey().$jsonRequestData);

        $header= [
            "Authorization: SHIRTIGO-CHECKSUM $checksum",
            'Content-type: application/json'
        ];

        curl_setopt($ch,CURLOPT_HTTPHEADER,$header);
        curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);
        curl_setopt($ch,CURLOPT_HEADER,true);
        curl_setopt($ch,CURLOPT_POSTFIELDS,$jsonRequestData);
        curl_setopt($ch,CURLOPT_POST,true);

        $result=curl_exec($ch);

        $httpCode=curl_getinfo($ch,CURLINFO_HTTP_CODE);

        if ($httpCode>=400) {

            if ($httpCode==404) {
                throw new RequestException('The URL '.$url.' is not reachable. Please try to reinstall the Cockpit-Plugin plugin and enter the config-data again.');
            }

            throw new RequestException('The request to the URL '.$url.' could not be executed successfully. HTTP-Code '.$httpCode. ' returned');
        }

        curl_close($ch);

        return $result;
    }

}