<?php

namespace ShirtigoCockpitPrintfulfillment\Components;

use Shopware\Components\Api\Exception\NotFoundException;
use Shopware\Components\HttpClient\RequestException;

class IntegrationCreatorComponent
{

    private $cockpitUser;

    private $cockpitPassword;

    private $shopwareAPIUser;

    private $formShopURL;

    const shopURLDefault='https://www.myshopwareshop.de';

    const referenceFile='/.reference';

    public function __construct($cockpitUser, $cockpitPassword, $apiUser, $formShopURL=self::shopURLDefault)
    {
        // Find User-Model

        $userData = Shopware()->Db()->query('SELECT id from s_core_auth where username=?', [$apiUser])->fetch();

        if (!$userData) {
            throw new NotFoundException('The User with the name '.$apiUser.' could not be found.');
        }

        $userId=$userData['id'];

        $shopwareAPIUser = Shopware()->Models()->find('Shopware\Models\User\User', $userId);

        if (empty($shopwareAPIUser->getApiKey())) {
            throw new NotFoundException('The User with the name '.$apiUser.' does not have an APIKey.');
        }

        $this->shopwareAPIUser=$shopwareAPIUser;
        $this->cockpitUser=$cockpitUser;
        $this->cockpitPassword=$cockpitPassword;

        // Add "https://", if not present yet. If we would not do this, the URL-validation would deny
	    // this request.

	    if (!preg_match('/^https:\/\//',$formShopURL,$matches)) {

		    $this->formShopURL='https://'.$formShopURL;

	    } else {

		    $this->formShopURL=$formShopURL;

	    }

    }

    public function createIntegration() {

        $dataArray= [
            'user'      => $this->cockpitUser,
            'password'  => $this->cockpitPassword,
            'apiUser'   => $this->shopwareAPIUser->getUsername(),
            'apiKey'    => $this->shopwareAPIUser->getApiKey(),
            'shop'      => (!empty($this->formShopURL) && $this->formShopURL!=self::shopURLDefault) ? $this->formShopURL : 'https://' . $_SERVER['HTTP_HOST']
        ];

        $response=$this->createPutRequest(Config::integrationCreationEndpoint,$dataArray);

        // If we get a response, the creation was successful, so we have to store the reference.

        file_put_contents(__DIR__.self::referenceFile,str_replace('"','',$response));

        return true;
    }


    private function createPutRequest($url, $data) {

        $ch=curl_init($url);

        curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);
        curl_setopt($ch,CURLOPT_POSTFIELDS,http_build_query($data));
        curl_setopt($ch,CURLOPT_CUSTOMREQUEST, "PUT");

        curl_setopt($ch,CURLOPT_SSL_VERIFYHOST,false);
        curl_setopt($ch,CURLOPT_SSL_VERIFYSTATUS,false);
        curl_setopt($ch,CURLOPT_SSL_VERIFYPEER,false);

        $result=curl_exec($ch);

        $httpCode=curl_getinfo($ch,CURLINFO_HTTP_CODE);

        if ($httpCode>=400) {

            if ($httpCode==404) {
                throw new RequestException('The URL '.$url.' is not reachable.');
            }

            throw new RequestException('The request to the URL '.$url.' could not be executed successfully. Message: '.$result);
        }


        curl_close($ch);

        return $result;
    }

}