<?php

namespace ShirtigoCockpitPrintfulfillment\Subscriber;

use Doctrine\DBAL\Exception\DatabaseObjectNotFoundException;
use Enlight\Event\SubscriberInterface;
use ShirtigoCockpitPrintfulfillment\Components\Config;
use Shopware\Components\Api\Exception\NotFoundException;
use Shopware\Components\Plugin\ConfigReader;
use ShirtigoCockpitPrintfulfillment\Components\FulfillmentCreatorComponent;
use ShirtigoCockpitPrintfulfillment\Components\IntegrationCreatorComponent;
use ShirtigoCockpitPrintfulfillment\ShirtigoCockpitPrintfulfillment;
use Shopware\Models\Attribute\Order;
use Shopware\Models\Shop\Shop;

class RouteSubscriber implements SubscriberInterface
{

    private $config;

    public static function getSubscribedEvents()
    {
        return [
            'Shopware\Models\Order\Order::postUpdate' => 'createFulfillmentDatabaseEntry',
            'Shopware_Controllers_Backend_Config::saveFormAction::after' => 'createIntegration'
        ];
    }


    public function __construct($pluginName, ConfigReader $configReader)
    {

        $this->config=$configReader->getByPluginName($pluginName);

    }

	/**
	 * Creates a new integration on Cockpit or gets the current reference
	 *
	 * @param \Enlight_Event_EventArgs $args
	 *
	 * @throws \Enlight_Exception
	 * @throws \ReflectionException
	 */
    public function createIntegration(\Enlight_Event_EventArgs $args) {

        $action = $args->getSubject();

        if (!$action) {
            return;
        }

        $request=$action->Request();

        if (!$request) {
            return;
        }

        $post=$request->getPost();

        if (!isset($post['label'])) {
            return;
        }

        $label=str_replace(' ','',$post['label']);

	        if ( $label == ShirtigoCockpitPrintfulfillment::PLUGIN_NAME ) {

		        $component = new IntegrationCreatorComponent( trim($this->config['ShirtigoCockpitPrintfulfillmentUser']),
			                                                  trim($this->config['ShirtigoCockpitPrintfulfillmentPassword']),
			                                                  trim($this->config['ShirtigoCockpitPrintfulfillmentAPIUser']),
			                                                  trim($this->config['ShirtigoCockpitPrintfulfillmentUrl']));

		        $component->createIntegration();

	        }

        return;

    }

	/**
	 * Creates an order-fulfillment for shirtigo-products when an order got the status "completely_paid".
	 *
	 * @param \Enlight_Event_EventArgs $args
	 *
	 * @throws NotFoundException
	 * @throws \Doctrine\ORM\ORMException
	 * @throws \Doctrine\ORM\OptimisticLockException
	 * @throws \Doctrine\ORM\TransactionRequiredException
	 * @throws \Exception
	 * @throws \Zend_Db_Adapter_Exception
	 * @throws \Zend_Db_Statement_Exception
	 */
    public function createFulfillmentDatabaseEntry(\Enlight_Event_EventArgs $args) {

    	// Get Order as model
        $entity=$args->get('entity');

        // Send request only if order is paid completely.

        if ($entity==null || $entity->getPaymentStatus()==null || $entity->getPaymentStatus()->getName()!='completely_paid') {
            return;
        }

        $details=$entity->getDetails();

        if ($details==null) {
            return;
        }

        foreach ($details as $num=>$detail) {

            $articleID = $detail->getArticleId();

            $article = Shopware()->Models()->find('Shopware\Models\Article\Article', $articleID);

            // If the article is (still) available on the system, we check if "Shirtigo Cockpit" is in the keywords.
            // If yes, we are making a request "back home", that the article can be sent.
            // For all cases where the article is not available any more, we make the request anyway, Shirtigo
            // will sort them out automatically.

            if ($article != null) {

                $supplier = $article->getSupplier();

                if ($supplier==null) {
                	continue;
                }

                $supplierName=$supplier->getName();

                if ($supplierName==null || trim($supplierName)!='Shirtigo Cockpit') {
                    continue;
                }
            }

            $em = Shopware()->Container()->get('models');

            $dataArray = $em->toArray($detail);

            if (isset($dataArray[0])) {
	            $dataArray = $dataArray[0];
            }

            if ($this->setAttributeJsonData($entity,json_encode($dataArray))) {

	            Shopware()->Container()->get('events')->notify(
		            'Shopware_CronJob_SC_Printfulfillment_Create_Fulfillment',
		            []
	            );
            }

            // It is enough to save & send the information of ONE ordered element. So we can stop here.

	        break;

        }
    }


    private function setAttributeJsonData($order,$jsonData) {

    	$orderId=$order->getId();
	    $dataLoadService=Shopware()->Container()->get('shopware_attribute.data_loader');

	    $existingAttribute=$dataLoadService->load('s_order_attributes',$orderId);

	    if (!empty($existingAttribute['shirtigopfjsonsent']) && $existingAttribute['shirtigopfjsonsent']==Config::STATUS_REQUEST_JSON_SENT) {

	    	    // Nothing to do
	    		return false;

	    }

    	$dataToWrite=$jsonData;

	    $persistService=Shopware()->Container()->get('shopware_attribute.data_persister');

	    $persistService->persist(['shirtigopfjsondata' => $dataToWrite, 'shirtigopfjsonsent' => Config::STATUS_REQUEST_JSON_READY],'s_order_attributes',$orderId);

	    return true;

}

}